@@ -1,67 +1,66 @@
-# -*- coding: utf-8 -*-
+# SPDX-License-Identifier: GPL-2.0-or-later
 #
 # Copyright (C) 2015-2016 Red Hat, Inc.
 #
 # Authors:
 # Thomas Woerner <twoerner@redhat.com>
-#
-# This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-#
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
-#
 
 """ipset io XML handler, reader, writer"""
 
-__all__ = [ "IPSet", "ipset_reader", "ipset_writer" ]
-
 import xml.sax as sax
 import os
 import io
 import shutil
 
 from firewall import config
-from firewall.functions import checkIP, checkIP6, checkIPnMask, \
-    checkIP6nMask, check_mac, check_port, checkInterface, \
-    checkProtocol
-from firewall.core.io.io_object import IO_Object, \
-    IO_Object_ContentHandler, IO_Object_XMLGenerator
+from firewall.functions import (
+    checkIP,
+    checkIP6,
+    checkIPnMask,
+    checkIP6nMask,
+    check_mac,
+    check_port,
+    checkInterface,
+    checkProtocol,
+)
+from firewall.core.io.io_object import (
+    IO_Object,
+    IO_Object_ContentHandler,
+    IO_Object_XMLGenerator,
+)
 from firewall.core.ipset import IPSET_TYPES, IPSET_CREATE_OPTIONS
-from firewall.core.icmp import check_icmp_name, check_icmp_type, \
-    check_icmpv6_name, check_icmpv6_type
+from firewall.core.icmp import (
+    check_icmp_name,
+    check_icmp_type,
+    check_icmpv6_name,
+    check_icmpv6_type,
+)
 from firewall.core.logger import log
 from firewall import errors
 from firewall.errors import FirewallError
 
+
 class IPSet(IO_Object):
     IMPORT_EXPORT_STRUCTURE = (
-        ( "version",  "" ),              # s
-        ( "short", "" ),                 # s
-        ( "description", "" ),           # s
-        ( "type", "" ),                  # s
-        ( "options", { "": "", }, ),     # a{ss}
-        ( "entries", [ "" ], ),          # as
+        ("version", ""),  # s
+        ("short", ""),  # s
+        ("description", ""),  # s
+        ("type", ""),  # s
+        ("options", {"": ""}),  # a{ss}
+        ("entries", [""]),  # as
     )
-    DBUS_SIGNATURE = '(ssssa{ss}as)'
-    ADDITIONAL_ALNUM_CHARS = [ "_", "-", ":", "." ]
+    DBUS_SIGNATURE = "(ssssa{ss}as)"
+    ADDITIONAL_ALNUM_CHARS = ["_", "-", ":", "."]
     PARSER_REQUIRED_ELEMENT_ATTRS = {
         "short": None,
         "description": None,
-        "ipset": [ "type" ],
-        "option": [ "name" ],
+        "ipset": ["type"],
+        "option": ["name"],
         "entry": None,
     }
     PARSER_OPTIONAL_ELEMENT_ATTRS = {
-        "ipset": [ "version" ],
-        "option": [ "value" ],
+        "ipset": ["version"],
+        "option": ["value"],
     }
 
     def __init__(self):
@@ -70,8 +69,8 @@ class IPSet(IO_Object):
         self.short = ""
         self.description = ""
         self.type = ""
-        self.entries = [ ]
-        self.options = { }
+        self.entries = []
+        self.options = {}
         self.applied = False
 
     def cleanup(self):
@@ -91,19 +90,19 @@ class IPSet(IO_Object):
                 family = "ipv6"
 
         if not ipset_type.startswith("hash:"):
-            raise FirewallError(errors.INVALID_IPSET,
-                                "ipset type '%s' not usable" % ipset_type)
+            raise FirewallError(
+                errors.INVALID_IPSET, "ipset type '%s' not usable" % ipset_type
+            )
         flags = ipset_type[5:].split(",")
         items = entry.split(",")
 
         if len(flags) != len(items) or len(flags) < 1:
             raise FirewallError(
                 errors.INVALID_ENTRY,
-                "entry '%s' does not match ipset type '%s'" % \
-                (entry, ipset_type))
+                "entry '%s' does not match ipset type '%s'" % (entry, ipset_type),
+            )
 
-        for i in range(len(flags)):
-            flag = flags[i]
+        for i, flag in enumerate(flags):
             item = items[i]
 
             if flag == "ip":
@@ -112,21 +111,24 @@ class IPSet(IO_Object):
                     if i > 1:
                         raise FirewallError(
                             errors.INVALID_ENTRY,
-                            "invalid address '%s' in '%s'[%d]" % \
-                            (item, entry, i))
+                            "invalid address '%s' in '%s'[%d]" % (item, entry, i),
+                        )
                     splits = item.split("-")
                     if len(splits) != 2:
                         raise FirewallError(
                             errors.INVALID_ENTRY,
-                            "invalid address range '%s' in '%s' for %s (%s)" % \
-                            (item, entry, ipset_type, family))
+                            "invalid address range '%s' in '%s' for %s (%s)"
+                            % (item, entry, ipset_type, family),
+                        )
                     for _split in splits:
-                        if (family == "ipv4" and not checkIP(_split)) or \
-                           (family == "ipv6" and not checkIP6(_split)):
+                        if (family == "ipv4" and not checkIP(_split)) or (
+                            family == "ipv6" and not checkIP6(_split)
+                        ):
                             raise FirewallError(
                                 errors.INVALID_ENTRY,
-                                "invalid address '%s' in '%s' for %s (%s)" % \
-                                (_split, entry, ipset_type, family))
+                                "invalid address '%s' in '%s' for %s (%s)"
+                                % (_split, entry, ipset_type, family),
+                            )
                 else:
                     # IPs with mask only allowed in the first
                     # position of the type
@@ -134,8 +136,9 @@ class IPSet(IO_Object):
                         if item == "0.0.0.0":
                             raise FirewallError(
                                 errors.INVALID_ENTRY,
-                                "invalid address '%s' in '%s' for %s (%s)" % \
-                                (item, entry, ipset_type, family))
+                                "invalid address '%s' in '%s' for %s (%s)"
+                                % (item, entry, ipset_type, family),
+                            )
                         if i == 0:
                             ip_check = checkIPnMask
                         else:
@@ -145,8 +148,9 @@ class IPSet(IO_Object):
                     if not ip_check(item):
                         raise FirewallError(
                             errors.INVALID_ENTRY,
-                            "invalid address '%s' in '%s' for %s (%s)" % \
-                            (item, entry, ipset_type, family))
+                            "invalid address '%s' in '%s' for %s (%s)"
+                            % (item, entry, ipset_type, family),
+                        )
             elif flag == "net":
                 if "-" in item:
                     # IP ranges only with plain IPs, no masks
@@ -154,89 +158,111 @@ class IPSet(IO_Object):
                     if len(splits) != 2:
                         raise FirewallError(
                             errors.INVALID_ENTRY,
-                            "invalid address range '%s' in '%s' for %s (%s)" % \
-                            (item, entry, ipset_type, family))
+                            "invalid address range '%s' in '%s' for %s (%s)"
+                            % (item, entry, ipset_type, family),
+                        )
                     # First part can only be a plain IP
-                    if (family == "ipv4" and not checkIP(splits[0])) or \
-                       (family == "ipv6" and not checkIP6(splits[0])):
+                    if (family == "ipv4" and not checkIP(splits[0])) or (
+                        family == "ipv6" and not checkIP6(splits[0])
+                    ):
                         raise FirewallError(
                             errors.INVALID_ENTRY,
-                            "invalid address '%s' in '%s' for %s (%s)" % \
-                            (splits[0], entry, ipset_type, family))
+                            "invalid address '%s' in '%s' for %s (%s)"
+                            % (splits[0], entry, ipset_type, family),
+                        )
                     # Second part can also have a mask
-                    if (family == "ipv4" and not checkIPnMask(splits[1])) or \
-                       (family == "ipv6" and not checkIP6nMask(splits[1])):
+                    if (family == "ipv4" and not checkIPnMask(splits[1])) or (
+                        family == "ipv6" and not checkIP6nMask(splits[1])
+                    ):
                         raise FirewallError(
                             errors.INVALID_ENTRY,
-                            "invalid address '%s' in '%s' for %s (%s)" % \
-                            (splits[1], entry, ipset_type, family))
+                            "invalid address '%s' in '%s' for %s (%s)"
+                            % (splits[1], entry, ipset_type, family),
+                        )
                 else:
                     # IPs with mask allowed in all positions, but no /0
                     if item.endswith("/0"):
-                        if not (family == "ipv6" and i == 0 and
-                                ipset_type == "hash:net,iface"):
+                        if not (
+                            family == "ipv6"
+                            and i == 0
+                            and ipset_type == "hash:net,iface"
+                        ):
                             raise FirewallError(
                                 errors.INVALID_ENTRY,
-                                "invalid address '%s' in '%s' for %s (%s)" % \
-                                (item, entry, ipset_type, family))
-                    if (family == "ipv4" and not checkIPnMask(item)) or \
-                       (family == "ipv6" and not checkIP6nMask(item)):
+                                "invalid address '%s' in '%s' for %s (%s)"
+                                % (item, entry, ipset_type, family),
+                            )
+                    if (family == "ipv4" and not checkIPnMask(item)) or (
+                        family == "ipv6" and not checkIP6nMask(item)
+                    ):
                         raise FirewallError(
                             errors.INVALID_ENTRY,
-                            "invalid address '%s' in '%s' for %s (%s)" % \
-                            (item, entry, ipset_type, family))
+                            "invalid address '%s' in '%s' for %s (%s)"
+                            % (item, entry, ipset_type, family),
+                        )
             elif flag == "mac":
                 # ipset does not allow to add 00:00:00:00:00:00
                 if not check_mac(item) or item == "00:00:00:00:00:00":
                     raise FirewallError(
                         errors.INVALID_ENTRY,
-                        "invalid mac address '%s' in '%s'" % (item, entry))
+                        "invalid mac address '%s' in '%s'" % (item, entry),
+                    )
             elif flag == "port":
                 if ":" in item:
                     splits = item.split(":")
                     if len(splits) != 2:
                         raise FirewallError(
-                            errors.INVALID_ENTRY,
-                            "invalid port '%s'" % (item))
+                            errors.INVALID_ENTRY, "invalid port '%s'" % (item)
+                        )
                     if splits[0] == "icmp":
                         if family != "ipv4":
                             raise FirewallError(
                                 errors.INVALID_ENTRY,
-                                "invalid protocol for family '%s' in '%s'" % \
-                                (family, entry))
-                        if not check_icmp_name(splits[1]) and not \
-                           check_icmp_type(splits[1]):
+                                "invalid protocol for family '%s' in '%s'"
+                                % (family, entry),
+                            )
+                        if not check_icmp_name(splits[1]) and not check_icmp_type(
+                            splits[1]
+                        ):
                             raise FirewallError(
                                 errors.INVALID_ENTRY,
-                                "invalid icmp type '%s' in '%s'" % \
-                                (splits[1], entry))
-                    elif splits[0] in [ "icmpv6", "ipv6-icmp" ]:
+                                "invalid icmp type '%s' in '%s'" % (splits[1], entry),
+                            )
+                    elif splits[0] in ["icmpv6", "ipv6-icmp"]:
                         if family != "ipv6":
                             raise FirewallError(
                                 errors.INVALID_ENTRY,
-                                "invalid protocol for family '%s' in '%s'" % \
-                                (family, entry))
-                        if not check_icmpv6_name(splits[1]) and not \
-                           check_icmpv6_type(splits[1]):
+                                "invalid protocol for family '%s' in '%s'"
+                                % (family, entry),
+                            )
+                        if not check_icmpv6_name(splits[1]) and not check_icmpv6_type(
+                            splits[1]
+                        ):
                             raise FirewallError(
                                 errors.INVALID_ENTRY,
-                                "invalid icmpv6 type '%s' in '%s'" % \
-                                (splits[1], entry))
-                    elif splits[0] not in [ "tcp", "sctp", "udp", "udplite" ] \
-                       and not checkProtocol(splits[0]):
+                                "invalid icmpv6 type '%s' in '%s'" % (splits[1], entry),
+                            )
+                    elif splits[0] not in [
+                        "tcp",
+                        "sctp",
+                        "udp",
+                        "udplite",
+                    ] and not checkProtocol(splits[0]):
                         raise FirewallError(
                             errors.INVALID_ENTRY,
-                            "invalid protocol '%s' in '%s'" % (splits[0],
-                                                               entry))
+                            "invalid protocol '%s' in '%s'" % (splits[0], entry),
+                        )
                     elif not check_port(splits[1]):
                         raise FirewallError(
                             errors.INVALID_ENTRY,
-                            "invalid port '%s'in '%s'" % (splits[1], entry))
+                            "invalid port '%s'in '%s'" % (splits[1], entry),
+                        )
                 else:
                     if not check_port(item):
                         raise FirewallError(
                             errors.INVALID_ENTRY,
-                            "invalid port '%s' in '%s'" % (item, entry))
+                            "invalid port '%s' in '%s'" % (item, entry),
+                        )
             elif flag == "mark":
                 if item.startswith("0x"):
                     try:
@@ -244,64 +270,73 @@ class IPSet(IO_Object):
                     except ValueError:
                         raise FirewallError(
                             errors.INVALID_ENTRY,
-                            "invalid mark '%s' in '%s'" % (item, entry))
+                            "invalid mark '%s' in '%s'" % (item, entry),
+                        )
                 else:
                     try:
                         int_val = int(item)
                     except ValueError:
                         raise FirewallError(
                             errors.INVALID_ENTRY,
-                            "invalid mark '%s' in '%s'" % (item, entry))
+                            "invalid mark '%s' in '%s'" % (item, entry),
+                        )
                 if int_val < 0 or int_val > 4294967295:
                     raise FirewallError(
                         errors.INVALID_ENTRY,
-                        "invalid mark '%s' in '%s'" % (item, entry))
+                        "invalid mark '%s' in '%s'" % (item, entry),
+                    )
             elif flag == "iface":
                 if not checkInterface(item) or len(item) > 15:
                     raise FirewallError(
                         errors.INVALID_ENTRY,
-                        "invalid interface '%s' in '%s'" % (item, entry))
+                        "invalid interface '%s' in '%s'" % (item, entry),
+                    )
             else:
-                raise FirewallError(errors.INVALID_IPSET,
-                                    "ipset type '%s' not usable" % ipset_type)
+                raise FirewallError(
+                    errors.INVALID_IPSET, "ipset type '%s' not usable" % ipset_type
+                )
 
-    def _check_config(self, config, item, all_config):
+    def _check_config(self, config, item, all_config, all_io_objects):
         if item == "type":
             if config not in IPSET_TYPES:
-                raise FirewallError(errors.INVALID_TYPE,
-                                    "'%s' is not valid ipset type" % config)
+                raise FirewallError(
+                    errors.INVALID_TYPE, "'%s' is not valid ipset type" % config
+                )
         if item == "options":
             for key in config.keys():
                 if key not in IPSET_CREATE_OPTIONS:
-                    raise FirewallError(errors.INVALID_IPSET,
-                                        "ipset invalid option '%s'" % key)
-                if key in [ "timeout", "hashsize", "maxelem" ]:
+                    raise FirewallError(
+                        errors.INVALID_IPSET, "ipset invalid option '%s'" % key
+                    )
+                if key in ["timeout", "hashsize", "maxelem"]:
                     try:
                         int_value = int(config[key])
                     except ValueError:
                         raise FirewallError(
                             errors.INVALID_VALUE,
-                            "Option '%s': Value '%s' is not an integer" % \
-                            (key, config[key]))
+                            "Option '%s': Value '%s' is not an integer"
+                            % (key, config[key]),
+                        )
                     if int_value < 0:
                         raise FirewallError(
                             errors.INVALID_VALUE,
-                            "Option '%s': Value '%s' is negative" % \
-                            (key, config[key]))
-                elif key == "family" and \
-                     config[key] not in [ "inet", "inet6" ]:
+                            "Option '%s': Value '%s' is negative" % (key, config[key]),
+                        )
+                elif key == "family" and config[key] not in ["inet", "inet6"]:
                     raise FirewallError(errors.INVALID_FAMILY, config[key])
 
-    def import_config(self, config):
+    def import_config(self, config, all_io_objects):
         if "timeout" in config[4] and config[4]["timeout"] != "0":
             if len(config[5]) != 0:
                 raise FirewallError(errors.IPSET_WITH_TIMEOUT)
         for entry in config[5]:
             IPSet.check_entry(entry, config[4], config[3])
-        super(IPSet, self).import_config(config)
+        super(IPSet, self).import_config(config, all_io_objects)
+
 
 # PARSER
 
+
 class ipset_ContentHandler(IO_Object_ContentHandler):
     def startElement(self, name, attrs):
         IO_Object_ContentHandler.startElement(self, name, attrs)
@@ -322,35 +357,39 @@ class ipset_ContentHandler(IO_Object_ContentHandler):
             if "value" in attrs:
                 value = attrs["value"]
 
-            if attrs["name"] not in \
-               [ "family", "timeout", "hashsize", "maxelem" ]:
+            if attrs["name"] not in ["family", "timeout", "hashsize", "maxelem"]:
+                raise FirewallError(
+                    errors.INVALID_OPTION, "Unknown option '%s'" % attrs["name"]
+                )
+            if self.item.type == "hash:mac" and attrs["name"] in ["family"]:
                 raise FirewallError(
                     errors.INVALID_OPTION,
-                    "Unknown option '%s'" % attrs["name"])
-            if self.item.type == "hash:mac" and attrs["name"] in [ "family" ]:
+                    "Unsupported option '%s' for type '%s'"
+                    % (attrs["name"], self.item.type),
+                )
+            if (
+                attrs["name"] in ["family", "timeout", "hashsize", "maxelem"]
+                and not value
+            ):
                 raise FirewallError(
                     errors.INVALID_OPTION,
-                    "Unsupported option '%s' for type '%s'" % \
-                    (attrs["name"], self.item.type))
-            if attrs["name"] in [ "family", "timeout", "hashsize", "maxelem" ] \
-               and not value:
-                raise FirewallError(
-                    errors.INVALID_OPTION,
-                    "Missing mandatory value of option '%s'" % attrs["name"])
-            if attrs["name"] in [ "timeout", "hashsize", "maxelem" ]:
+                    "Missing mandatory value of option '%s'" % attrs["name"],
+                )
+            if attrs["name"] in ["timeout", "hashsize", "maxelem"]:
                 try:
                     int_value = int(value)
                 except ValueError:
                     raise FirewallError(
                         errors.INVALID_VALUE,
-                        "Option '%s': Value '%s' is not an integer" % \
-                        (attrs["name"], value))
+                        "Option '%s': Value '%s' is not an integer"
+                        % (attrs["name"], value),
+                    )
                 if int_value < 0:
                     raise FirewallError(
                         errors.INVALID_VALUE,
-                        "Option '%s': Value '%s' is negative" % \
-                        (attrs["name"], value))
-            if attrs["name"] == "family" and value not in [ "inet", "inet6" ]:
+                        "Option '%s': Value '%s' is negative" % (attrs["name"], value),
+                    )
+            if attrs["name"] == "family" and value not in ["inet", "inet6"]:
                 raise FirewallError(errors.INVALID_FAMILY, value)
             if attrs["name"] not in self.item.options:
                 self.item.options[attrs["name"]] = value
@@ -363,11 +402,13 @@ class ipset_ContentHandler(IO_Object_ContentHandler):
         if name == "entry":
             self.item.entries.append(self._element)
 
+
 def ipset_reader(filename, path):
     ipset = IPSet()
     if not filename.endswith(".xml"):
-        raise FirewallError(errors.INVALID_NAME,
-                            "'%s' is missing .xml suffix" % filename)
+        raise FirewallError(
+            errors.INVALID_NAME, "'%s' is missing .xml suffix" % filename
+        )
     ipset.name = filename[:-4]
     ipset.check_name(ipset.name)
     ipset.filename = filename
@@ -384,16 +425,20 @@ def ipset_reader(filename, path):
         try:
             parser.parse(source)
         except sax.SAXParseException as msg:
-            raise FirewallError(errors.INVALID_IPSET,
-                                "not a valid ipset file: %s" % \
-                                msg.getException())
+            raise FirewallError(
+                errors.INVALID_IPSET, "not a valid ipset file: %s" % msg.getException()
+            )
     del handler
     del parser
-    if "timeout" in ipset.options and ipset.options["timeout"] != "0" and \
-       len(ipset.entries) > 0:
+    if (
+        "timeout" in ipset.options
+        and ipset.options["timeout"] != "0"
+        and len(ipset.entries) > 0
+    ):
         # no entries visible for ipsets with timeout
-        log.warning("ipset '%s': timeout option is set, entries are ignored",
-                    ipset.name)
+        log.warning(
+            "ipset '%s': timeout option is set, entries are ignored", ipset.name
+        )
         del ipset.entries[:]
     i = 0
     entries_set = set()
@@ -414,6 +459,7 @@ def ipset_reader(filename, path):
 
     return ipset
 
+
 def ipset_writer(ipset, path=None):
     _path = path if path else ipset.path
 
@@ -434,12 +480,12 @@ def ipset_writer(ipset, path=None):
             os.mkdir(config.ETC_FIREWALLD, 0o750)
         os.mkdir(dirpath, 0o750)
 
-    f = io.open(name, mode='wt', encoding='UTF-8')
+    f = io.open(name, mode="wt", encoding="UTF-8")
     handler = IO_Object_XMLGenerator(f)
     handler.startDocument()
 
     # start ipset element
-    attrs = { "type": ipset.type }
+    attrs = {"type": ipset.type}
     if ipset.version and ipset.version != "":
         attrs["version"] = ipset.version
     handler.startElement("ipset", attrs)
@@ -448,7 +494,7 @@ def ipset_writer(ipset, path=None):
     # short
     if ipset.short and ipset.short != "":
         handler.ignorableWhitespace("  ")
-        handler.startElement("short", { })
+        handler.startElement("short", {})
         handler.characters(ipset.short)
         handler.endElement("short")
         handler.ignorableWhitespace("\n")
@@ -456,30 +502,30 @@ def ipset_writer(ipset, path=None):
     # description
     if ipset.description and ipset.description != "":
         handler.ignorableWhitespace("  ")
-        handler.startElement("description", { })
+        handler.startElement("description", {})
         handler.characters(ipset.description)
         handler.endElement("description")
         handler.ignorableWhitespace("\n")
 
     # options
-    for key,value in ipset.options.items():
+    for key, value in ipset.options.items():
         handler.ignorableWhitespace("  ")
         if value != "":
-            handler.simpleElement("option", { "name": key, "value": value })
+            handler.simpleElement("option", {"name": key, "value": value})
         else:
-            handler.simpleElement("option", { "name": key })
+            handler.simpleElement("option", {"name": key})
         handler.ignorableWhitespace("\n")
 
     # entries
     for entry in ipset.entries:
         handler.ignorableWhitespace("  ")
-        handler.startElement("entry", { })
+        handler.startElement("entry", {})
         handler.characters(entry)
         handler.endElement("entry")
         handler.ignorableWhitespace("\n")
 
     # end ipset element
-    handler.endElement('ipset')
+    handler.endElement("ipset")
     handler.ignorableWhitespace("\n")
     handler.endDocument()
     f.close()
